//
//  HwRtcEngine.h
//  HwRtcEngine
//
//  Created by zhanganl on 2020/2/24.
//  Copyright © 2020 zhang. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "HwConstants.h"
#import "HwObject.h"

@protocol HwRtcEngineDelegate <NSObject>
@optional

/// 加入房间成功回调
/// @param roomId 房间ID
/// @param userId 用户ID
-(void)rtcEngineJoinSuccessCallback:(NSString*)roomId userId:(NSString*)userId;

/// 加入房间失败
- (void)rtcEngineJoinRoomFailure;

/// 离开房间完成回调
-(void)rtcEngineLeaveRoomCallback;


/// 用户加入房间回调
/// @param roomId 房间ID
/// @param userId 用户ID
/// @param nickName 昵称
-(void)rtcEngineUserJoinedCallback:(NSString*)roomId userId:(NSString*)userId nickName:(NSString*)nickName;

/// 用户离开房间回调
/// @param roomId 房间ID
/// @param userId 用户ID
/// @param reason 离开原因
-(void)rtcEngineUserOfflineCallback:(NSString*)roomId userId:(NSString*)userId reason:(NSInteger)reason;

/// 第一个输入用户
/// @param userId 用户ID
/// @param width 宽
/// @param height 高
-(void)rtcEngineFirstRemoteVideoDecodedCallback:(NSString*)userId width:(int)width height:(int)height;

/// 默认流的分辨率
/// @param roomid 默认流房间
/// @param userid 用户id
/// @param streamType 流支持选看的类型 HwRtcStreamType
- (void)rtcEngineStreamAvailableWithRoomid:(NSString *)roomid userid:(NSString *)userid streamType:(HwRtcStreamType)streamType;

/// 调用setUserRole触发的回调
/// @param newRole 新的角色
/// @param oldRole 旧的角色
- (void)rtcEngineUserRoleChangeWithNewRole:(HwRtcRole)newRole oldRole:(HwRtcRole)oldRole;

/// 引擎状态
/// @param state 链接服务器状态 可参考HwRtcConnStateType
/// @param reason 引擎状态 可参考HwRtcConnChangeReason
/// @param description 相对应引擎状态解释
-(void)rtcEngineConnStateChangeCallback:(HwRtcConnStateType)state reason:(HwRtcConnChangeReason)reason description:(NSString*)description;

/// 出错回调
/// @param errorCode 错误码
/// @param errorMsg <#errorMsg description#>
-(void)rtcEngineErrorCallback:(int)errorCode errorMsg:(NSString*)errorMsg;

/// 警告回调
-(void)rtcEngineWarningCallback;

#pragma mark - screenChapture

- (void)rtcEngineSubStreamAvailableWitRoomId:(NSString *)roomId userId:(NSString *)userId isAvailable:(BOOL)isAvailable;


#pragma mark - Device

/// 设备状态发生改变
/// @param deviceId 设备ID
/// @param deviceType 设备类型
/// @param deviceState 设备状态
-(void)rtcEngineDeviceStateChangedCallback:(NSString*)deviceId deviceType:(HwRtcDeviceType)deviceType deviceState:(HwRtcDeviceState)deviceState;

/// 音频设备音量发生改变
/// @param deviceType 音频设备类型
/// @param volume 音量值 0-100
-(void)rtcEngineAudioVolumeChangedCallback:(HwRtcDeviceType)deviceType volume:(unsigned int)volume;

/// 音频路由发生改变(只支持iOS)
/// @param audioRoute 音频路由类型 参考HwRtcAudioRoute
- (void)rtcEngineAudioRouteChange:(HwRtcAudioRoute)audioRoute;

#pragma mark - Information
/// 上传日志成功状态回调
/// @param result 0 表示上传日志成功 1表示上传日志失败
-(void)rtcEngineLogUploadResultCallback:(int)result;

/// 上传日志进度条回调
/// @param progress int 进度值 0 - 100
-(void)rtcEngineLogUploadProgressCallback:(int)progress;

/// 每隔五秒返回视频信息
/// @param localVideoStatsInfos 本地视频信息
/// @param remoteVideoStatsInfos 远端视频信息
- (void)rtcEngineLocalVideoInfo:(NSArray <HwRtcVideoStatsInfo *> *)localVideoStatsInfos remoteVideoInfo:(NSArray <HwRtcVideoStatsInfo *>*)remoteVideoStatsInfos;

/// 每隔五秒返回音频信息
/// @param localAudioStatsInfos 本地音频信息
/// @param remoteAudioStatsInfos 远端视频信息
- (void)rtcEngineLocalAudioInfo:(NSArray <HwRtcAudioStatsInfo *> *)localAudioStatsInfos remoteAudioInfo:(NSArray <HwRtcAudioStatsInfo *> *)remoteAudioStatsInfos;

/// 辅流信息
/// @param localVideoStatsInfo 本地视频信息 如果是本地共享则改参数有值,否则为nil
/// @param remoteVideoStatsInfo 远端视频信息 如果是远端共享则该参数有值,否则为nil
- (void)rtcEngineSubStreamInfoWithLocalVideoInfo:(HwRtcVideoStatsInfo *)localVideoStatsInfo remoteVideoInfo:(HwRtcVideoStatsInfo *)remoteVideoStatsInfo;
@end


@interface HwRtcEngine :NSObject
@property (nonatomic, weak) id<HwRtcEngineDelegate> delegate;

/** 引擎单列话.保证APP只有一个引擎对象.
@return HwRtcEngine 对象.
*/
+ (instancetype)sharedEngine;

/// 引擎初始化配置信息,
/// 如果domain 和appid发生改变 需要调用 destroy
/// @param domain 服务器地址
/// @param appId appid
-(int)init:(NSString*)domain appId:(NSString*)appId;

/// /// 设置打印日志级别和日志地址
/// @param logLevel 打印级别.HwRtcLogLevel
///                 ENGINE_LOG_LEVEL_ERROR = 0,
///                 ENGINE_LOG_LEVEL_WARNING,
///                 ENGINE_LOG_LEVEL_INFO,
///                 ENGINE_LOG_LEVEL_DEBUG
/// @param logPath 日志地址
/// @param enable 是否开启日志
-(int)setLogParam:(HwRtcLogLevel)logLevel logPath:(NSString*)logPath enable:(BOOL)enable;

/// 加入房间
/// @param userInfo 用户信息 参考HwRtcUserInfo
/// @param roomId 房间ID
/// @param mediaType HwRtcMediaType
/// @ctime ctime 当前时间戳+2小时的失效时间
-(int)joinRoom:(HwRtcUserInfo *)userInfo roomId:(NSString*)roomId mediaType:(HwRtcMediaType)mediaType;

/// 设置本地视频编码
/// @param videoEncoder HwRtcVideoEncode 对象.携带编码格式和总带宽
- (int)setVideoEncoder:(HwRtcVideoEncode *)videoEncoder;

/// 角色切换
/// @param role 可以切换的模式 HwRtcRoleJoiner 和 HwRtcRolePlayer 直接切换.无法切换到HwRtcRolePublisher
- (int)setUserRole:(HwRtcRole)role;

// 设置本地预览画面/取消本地与看画面
/// @param local 预览试图 具体参考HwRtcVideoCanvas
/// @param viewMode 显示模式 具体参考HwRtcVideoDisplayMode
-(int)setupLocalVideo:(HwRtcVideoCanvas *)local viewMode:(HwRtcVideoDisplayMode)viewMode;

/// 设置远端预览画面
/// @param remote 预览试图 具体HwRtcVideoCanvas
/// @param viewMode 显示模式 具体参考HwRtcVideoDisplayMode
/// @param streamType 流清晰度 具体参考 HwRtcStreamType
-(int)setupRemoteView:(HwRtcVideoCanvas *)remote viewMode:(HwRtcVideoDisplayMode)viewMode streamType:(HwRtcStreamType)streamType;

/// 设置远端共享画面
/// @param streamView 远端共享试图 具体参考HwRtcVideoCanvas
/// @param viewMode 显示模式 具体参考HwRtcVideoDisplayMode
/// @param streamType 流清晰度 具体参考 HwRtcStreamType
-(int)setupRemoteSubStreamView:(HwRtcVideoCanvas *)streamView viewMode:(HwRtcVideoDisplayMode)viewMode streamType:(HwRtcStreamType)streamType;

/// 开启预览
/// 要调用setupLocalVideo 才会生效.
/// 在[joinRoom:roomId:mediaType:]之前调用才会生效
/// @return int 0表示成功.,其他表示失败
- (int)startPreview;

/// 关闭预览
/// 在[joinRoom:roomId:mediaType:]之前调用才会生效.如果开启预览之后加入房间,无需再调用关闭预览,离开房间会自动释放摄像头
/// @return int 0表示成功.,其他表示失败
- (int)stopPreview;

/// 只支持IPHONE
/// 切换前后镜头
-(int)switchCamera;

/// 设置扬声器模式 (只支持IPHONE)
/// @param speakerModel 扬声器模式类型 具体参考HwRtcSpeakerModel
-(int)setSpeakerModel:(HwRtcSpeakerModel)speakerModel;

/// 是否对本地静音
/// @param mute BOOL类型 YES 表示对本地静音 NO 表示对本地不静音
-(int)muteLocalAudio:(BOOL)mute;

/// 是否关闭本地预览页面
/// @param mute BOOL 类型 YES表示关闭本地预览页面 NO表示开启本地预览页面
-(int)muteLocalVideo:(BOOL)mute;

/// 是否关闭输入声音音量
/// @param mute BOOL 类型 YES表示输入声音静音, NO表示输入声音不静音
-(int)muteRemoteAudio:(BOOL)mute;

/// 调整采集音量
/// @param volume 音量值(0-100)
-(int)adjustRecordingVolume:(unsigned int)volume;

/// 调整播放音量
/// @param volume 音量值(0-100)
-(int)adjustPlaybackVolume:(unsigned int)volume;

/// 获取输入声音是否静音
-(BOOL)getRemoteAudioMute;

/// 获取输出声音是否静音
-(BOOL)getLocalAudioMute;

/// 设置扬声器的音量 (只支持MacOS)
/// @param volume int类型 取值在0-100, 10位正常音量,0为静音,100为最大音量
-(int)setPlaybackDevicesVolume:(unsigned int)volume;

/// 获取扬声器的音量 (只支持MacOS)
/// @return int
-(int)getPlaybackDevicesVolume;

/// 设置输入声音音量 (只支持MacOS)
/// @param volume 设置音量大小 volume取值范围是[0 100],10表示正常音量
-(int)setRecordingDevicesVolume:(unsigned int)volume;

/// 获取输入声音音量 (只支持MacOS)
-(int)getRecordingDevicesVolume;

/// 设置输出声音设备是否静音 (只支持MacOS)
/// @param mute BOOL YES 表示输出声音设备静音, NO 表示输出声音设备不静音
-(int)setplaybackDeviceMute:(BOOL)mute;

/// 是否关闭麦克风 (只支持MacOS)
/// @param mute BOOL 类型 YES 表示关闭麦克风 NO表示开启麦克风
-(int)setRecordingDeviceMute:(BOOL)mute;

/** 获取扬声器设备 (只支持MacOS)

@return (NSArray<HwRtcDeviceInfo*>*) 数组类型,数组中的对象HwRtcDeviceInfo类型的.
* nil: Failure.
*/
-(NSArray<HwRtcDeviceInfo*>*)getPlaybackDevices;

/** 获取麦克风设备 (只支持MacOS)
@return (NSArray<HwRtcDeviceInfo*>*) 数组类型,数组中的对象HwRtcDeviceInfo类型的.
* nil: Failure.
*/
-(NSArray<HwRtcDeviceInfo*>*)getRecordingDevices;

/** 获取摄像头设备 (只支持MacOS)
@return (NSArray<HwRtcDeviceInfo*>*) 数组类型,数组中的对象HwRtcDeviceInfo类型的.
* nil: Failure.
*/
-(NSArray<HwRtcDeviceInfo*>*)getVideoDevices;

/// 设置当前扬声器设备 (只支持MacOS)
/// @param deviceId 设备ID,可以从GetPlaybackDevices 获取到设备ID
-(int)setPlaybackDevice:(NSString*)deviceId;

/// 设置当前麦克风设备 (只支持MacOS)
/// @param deviceId 设备ID 可以从GetRecordingDevices 获取到设备ID
-(int)setRecordingDevice:(NSString*)deviceId;

/// 设置当前摄像头设备 (只支持MacOS)
/// @param deviceId 设备ID 可以从GetVideoDevices 获取到设备ID
-(int)setVideoDevice:(NSString*)deviceId;

/// 获取当前输入声音设备 (只支持MacOS)
/// 该值有可能为空.用户设置了什么,就返回什么.
- (NSString *)getCurrentRecordingDevice;

/// 获取当前输出声音设备 (只支持MacOS)
/// 该值有可能为空.用户设置了什么,就返回什么.
- (NSString *)getCurrentPlaybackDevice;

/// 获取当前摄像头设备 (只支持MacOS)
/// 该值有可能为空.用户设置了什么,就返回什么.
- (NSString *)getCurrentVideoDevice;

/// 日志上传
- (int)logUpload;

/// 离开房间
- (int)leaveRoom;

/// 注销引擎
- (void)destroy;
@end
